#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define ASCII_TABLE     256
#define FILE_SIZE       10240

int generate_ascii_table(FILE *out)
{
    unsigned char data[ASCII_TABLE];

    if (out == NULL) return 0;
    for (int i = 0; i < ASCII_TABLE; i ++)
        data[i] = i;
    return fwrite(data, ASCII_TABLE, 1, out) > 0;
}

int generate_sample_text(FILE *out)
{
    if (out == NULL) return 0;
    fprintf(out, "Aupres De Mon Arbre\nGeorges Brassens\n\n");
    return fprintf(out,
            "J'ai plaqué mon chêne\n"
            "Comme un saligaud,\n"
            "Mon copain le chêne,\n"
            "Mon alter ego,\n"
            "On était du même bois\n"
            "Un peu rustique, un peu brut,\n"
            "Dont on fait n'importe quoi\n"
            "Sauf, naturell'ment, les flûtes...\n"
            "J'ai maint'nant des frênes,\n"
            "Des arbres de Judée,\n"
            "Tous de bonne graine,\n"
            "De haute futaie...\n"
            "Mais, toi, tu manque' à l'appel,\n"
            "Ma vieill' branche de campagne,\n"
            "Mon seul arbre de Noël,\n"
            "Mon mât de cocagne !\n\n"
            "Auprès de mon arbre,\n"
            "Je vivais heureux,\n"
            "J'aurais jamais dû m'éloigner de mon arbre...\n"
            "Auprès de mon arbre,\n"
            "Je vivais heureux,\n"
            "J'aurais jamais dû le quitter des yeux...\n"
           ) > 0;
}

int last_index_under(int value, int length, int array[])
{
    for (int i = 0; i < length; i ++)
        if (array[i] > value)
            return i;
    return -1;
}

int generate_text_with_probability(FILE* out, int text_len, int frequency[])
{
    int table[ASCII_TABLE];
    table[0] = frequency[0];
    for (int i = 1; i < ASCII_TABLE; i++)
        table[i] = table[i - 1] + frequency[i];
    int max = table[ASCII_TABLE - 1];

    while (text_len-- > 0) {
        int c = last_index_under(rand()%max, ASCII_TABLE, table);
        if (fputc(c, out) == EOF)
            return 0;
    }
    return 1;
}

void exit_if(int condition, const char *prefix) {
    if (condition) {
        perror(prefix);
        exit(EXIT_FAILURE);
    }
}

int main(int argc, char *argv[])
{
    srand(argc > 1 ? atoi(argv[1]) : time(NULL));

    // it's bad, none of these file are closed ...

    exit_if(
        generate_ascii_table(fopen("ascii.txt", "w")) == 0, "ascii.txt");
    exit_if(
        generate_sample_text(fopen("arbre.txt", "w")) == 0, "arbre.txt");

    int table[ASCII_TABLE] = {0};

    for (int i = 32; i < 127; i ++)
        table[i] = rand() % 100 + 1;
    exit_if(
        generate_text_with_probability(fopen("well.txt", "w"), FILE_SIZE, table) == 0, "well.txt");

    for (int i = 0; i < ASCII_TABLE; i ++)
        table[i] = 1;
    exit_if(
        generate_text_with_probability(fopen("bad.txt", "w"), FILE_SIZE, table) == 0, "bad.txt");

    for (int i = 0; i < ASCII_TABLE; i ++)
        table[i] = rand() % 256;
    exit_if(
        generate_text_with_probability(fopen("another.txt", "w"), FILE_SIZE, table) == 0, "another.txt");

    return 0;
}
